/* ---------------------------------------------------------------------------------
 *  Copyright (C) 2006-2007 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------------
 */

/***********************************************************************************
 *    	Module Name : DS8007 Basic Input and Output functions
 *    	Description : This module contains function entry points for reading, writing
 *                    ds8007 registers and sending T1 DataBlock to ds8007.
 *                    This module has been written for DS5002 microcontroller to increase
 *                    read and write performance and meet EMV CWT timing
 *                    
 *         Filename : ds8007_io.a51
 *  Dependant Files : common.h
 *    	     Author : SGN
 *    	   Compiler : keil C51 Compiler V8.05a
 *    	    Version : Version 1.00.00
 *    	    Created : 
 *    Modifications : 
 *     	    	      
 *    	      Notes :
 **********************************************************************************/

$include (common.h)

#ifdef DS5002

$NOMOD51
$include (reg5000.h)

NAME	DS8007_IO

?PR?_dssc_readregister?LIB8007           SEGMENT CODE 
?PR?_dssc_writeregister?LIB8007          SEGMENT CODE 
?PR?_writeBlock?LIB8007                  SEGMENT CODE 
?XD?_writeBlock?LIB8007                  SEGMENT XDATA 
	PUBLIC	?_writeBlock?BYTE
	PUBLIC	_writeBlock


	PUBLIC	_dssc_writeregister
	PUBLIC	_dssc_readregister

	EXTRN	CODE (?C?CSTPTR)
	EXTRN	CODE (?C?CLDPTR)
	EXTRN	CODE (?C?CCASE)
 
	RSEG  ?XD?_writeBlock?LIB8007
?_writeBlock?BYTE:
        NAD?546:   DS   1
	ORG  1
        PCB?547:   DS   1
	ORG  2
     length?548:   DS   1
	ORG  3
     buffer?549:   DS   3
	ORG  6
        EDC?550:   DS   1
	ORG  7
          i?551:   DS   1

/*
*  Read data byte from DS8007 register
*  input: r7 - contains register number
*  output: r7 - contains databyte value
*/
   
	RSEG  ?PR?_dssc_readregister?LIB8007
_dssc_readregister:
	USING	0
    PUSH    IE
	CLR  	EA

	ORL  	RPCTL,#020H
 
    MOV  	A,R7
    push    ar7
	
    LCALL	?C?CCASE
	DW   	?C0239
	DB   	0DH
	DW   	00H
	DW   	?C0237
?C0238:
?C0239:
	MOV  	R3,#01H
	MOV  	R2,#00H
	MOV  	R1,#0CH
	LCALL	?C?CLDPTR
	MOV  	R7,A
	JB   	ACC.4,?C0237
	MOV  	R3,#01H
	MOV  	R2,#00H
	MOV  	R1,#07H
	LCALL	?C?CLDPTR
	MOV  	R7,A
	JB   	ACC.0,?C0239
	ANL  	RPCTL,#0DFH
    pop     ar7
	POP     IE
	MOV  	R7,#00H
	RET  	
?C0241:
	SJMP 	?C0239
?C0240:
?C0243:
?C0237:
    POP     aR7
	MOV  	R6,#00H
	MOV  	R1,AR7
	MOV  	R2,AR6
	MOV  	R3,#01H
	LCALL	?C?CLDPTR
	MOV  	R7,A
	ANL  	RPCTL,#0DFH
	POP     IE
	RET  	
; END OF _dssc_readregister

dssc_readMSRregister:

	ORL  	RPCTL,#020H
	MOV  	R3,#01H
	MOV  	R2,#00H
	MOV  	R1,#0CH
	LCALL	?C?CLDPTR
   	ANL  	RPCTL,#0DFH
	MOV  	R7,A
	RET  	

dssc_readUSRregister:

	ORL  	RPCTL,#020H
	MOV  	R3,#01H
	MOV  	R2,#00H
	MOV  	R1,#0EH
	LCALL	?C?CLDPTR
   	ANL  	RPCTL,#0DFH
	MOV  	R7,A
	RET  	


; 
; void dssc_writeregister(uint8_t address, uint8_t value)

/*
*  writes data byte to DS8007 register
*  input: r7 - contains register number
*         ar3:ar2:ar1 - address of ds8007 register
*/


	RSEG  ?PR?_dssc_writeregister?LIB8007
_dssc_writeregister:
	USING	0
	PUSH    IE
    CLR  	EA
	ORL  	RPCTL,#020H
    PUSH    aR7
    PUSH    aR5
	MOV  	A,R7
	LCALL	?C?CCASE
	DW   	?C0247
	DB   	08H
	DW   	?C0247
	DB   	0DH
	DW   	00H
	DW   	?C0244
?C0245:
?C0246:
?C0247:
	MOV  	R3,#01H
	MOV  	R2,#00H
	MOV  	R1,#0CH
	LCALL	?C?CLDPTR
	MOV  	R7,A
	JNB  	ACC.4,?C0247
?C0248:
?C0249:
?C0244:

    POP     aR7  ;read value
    POP     aR5  ;read address

	MOV  	R4,#00H
	MOV  	R1,AR5
	MOV  	R2,AR4
	MOV  	R3,#01H
	MOV  	A,R7
	LCALL	?C?CSTPTR
	ANL  	RPCTL,#0DFH
	POP     IE
?C0250:
	RET  	


/*
*  Internal function writes data byte to UTR register
*/

dssc_writeUTRregister:

	ORL  	RPCTL,#020H
    PUSH    aR5
?C02471:
	MOV  	R3,#01H
	MOV  	R2,#00H
	MOV  	R1,#0CH
	LCALL	?C?CLDPTR
	JNB  	ACC.4,?C02471

    POP     aR7  ;read value

	MOV  	R1,#0Dh
	MOV  	R2,#0
	MOV  	R3,#01H
	MOV  	A,R7
	LCALL	?C?CSTPTR
	ANL  	RPCTL,#0DFH
	RET  	


/*
 *extern void writeBlock(uint8_t NAD, uint8_t PCB, uint8_t length, uint8_t *buffer,uint8_t EDC);
 *
 *  Writes T1 Block to Ds8007 
 *
 *  Input : 
 *  ;R7 will hold NAD
 *  ;R5 will hold PCB
 *  ;R3 will hold length
 *   buffer address will be stored in  #buffer?549 location and EDC will be stored in  #EDC?550
 */

	RSEG  ?PR?_writeBlock?LIB8007
_writeBlock:
	USING	0
;R7 will hold NAD
;R5 will hold PCB
;R3 will hold length

	PUSH    IE
    clr     EA

;  	dssc_writeregister(UTR, EDC);
	MOV  	DPTR,#EDC?550
	MOVX 	A,@DPTR
    
    PUSH    ACC ;; store EDC value in stack
    push    aR3  ;;store length in stack
    push    aR5  ;;store  PCB value in stack

;  	dssc_writeregister(NAD, EDC);
	MOV  	R5,aR7
	LCALL	dssc_writeUTRregister
?C0026:
; 	while((dssc_readregister(MSR)&0x1)!=1);
			; SOURCE LINE # 288
	LCALL	dssc_readMSRregister
	JB  	ACC.0, ?C0027

    LCALL   dssc_readUSRregister

    JB      ACC.1,?C0027

    JMP     ?C0026
?C0027:

; 
;  	dssc_writeregister(UTR, PCB);
    POP     aR5
	LCALL	dssc_writeUTRregister
?C0028:
; 	while((dssc_readregister(MSR)&0x1)!=1);
			; SOURCE LINE # 291
	LCALL	dssc_readMSRregister
	JB  	ACC.0,?C0029

    LCALL   dssc_readUSRregister

    JB      ACC.1,?C0029
    JMP     ?C0028

?C0029:
; 
;  	dssc_writeregister(UTR, length);
    POP     aR5
    PUSH    aR5  ;store length into stack back. we dont know whether writeregister retains it
	LCALL	dssc_writeUTRregister

    pop     ar3

    cjne r3,#0, send_inf_data
    jmp  send_edc

send_inf_data:
	MOV  	DPTR,#buffer?549
    inc     dptr
    movx   a,@dptr
    push    acc
    inc     dptr
    movx   a,@dptr
    mov     dpl,acc
    pop     dph

send_inf_data1:

    push    ar3	
    push    dpl
    push    dph

?C0030:
; 	while((dssc_readregister(MSR)&0x1)!=1);
			; SOURCE LINE # 294
	LCALL	dssc_readMSRregister
	;JNB  	ACC.0,?C0030

	JB  	ACC.0,?C0031

    LCALL   dssc_readUSRregister

    JB      ACC.1,?C0031
    JMP     ?C0030


?C0031:

    POP    DPH
    POP    DPL

    push    dpl
    push    dph

	MOVX 	A,@DPTR
;	MOV  	R5,#0FEh
    mov     r5,acc
    clr     ti
	LCALL	dssc_writeUTRregister

    POP    DPH
    POP    DPL
    POP    aR3

    inc    dptr
    
    djnz  r3, send_inf_data1


?C0036:

?C0033:
send_edc:
?C0035:

; 	while((dssc_readregister(MSR)&0x1)!=1);
			; SOURCE LINE # 303
	LCALL	dssc_readMSRregister
	;JNB  	ACC.0,?C0035


	JB  	ACC.0,write_edc_to_utr

    LCALL   dssc_readUSRregister

    JB      ACC.1,write_edc_to_utr

    JMP     ?C0035

write_edc_to_utr:
    POP     aR5
	LCALL	dssc_writeUTRregister
?C0037:
	LCALL	dssc_readMSRregister

	JB  	ACC.0,?C0038
    LCALL   dssc_readUSRregister
    JB      ACC.1,?C0038
    JMP    ?C0037

?C0038:
    POP   IE  
?C0039:
	RET  	
; END OF _writeBlock

#endif
 
end
